<?php

class BinEventCalendar
{
	function __construct()
	{
		// translation
		add_action('init', [$this, 'load_textdomain']);
		
		// add styles/scripts
		add_action('admin_enqueue_scripts', [$this, 'admin_enqueue_scripts']);
		add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);
		
		// custompages
		add_action('init', [$this, 'init_custompages']);
		add_filter('query_vars', [$this, 'query_vars_custompages']);
		add_action('parse_request', [$this, 'parse_request_custompages']);
		
		// dependency check
		//add_action('wp_ajax_bin_event_calendar_admin_notice_dismiss', [$this, 'wp_ajax_bin_event_calendar_admin_notice_dismiss']);
		//if(get_option('bin_event_calendar_dependency_check') === 'enabled') { add_action('admin_notices', [$this, 'admin_notice']); }
	}
	
	/**
	 * load textdomain
	 */
	function load_textdomain()
	{
		// vars
		$domain = 'bin-event-calendar';
		$locale = is_admin() && function_exists('get_user_locale') ? get_user_locale() : get_locale();
		$mofile = $domain.'-'.$locale.'.mo';

		// load from the languages directory first
		load_textdomain($domain, WP_LANG_DIR.'/plugins/'.$mofile);

		// redirect missing translations
		$mofile = str_replace('de_DE', 'de_CH', $mofile);

		// load from plugin lang folder
		$loaded = load_textdomain($domain, BIN_EVENT_CALENDAR_DIR.'/languages/'.$mofile);
	}
	
	/**
	 * Add admin style/script
	 */
	function admin_enqueue_scripts()
	{
		wp_enqueue_style('bin-event-calendar-back', BIN_EVENT_CALENDAR_URL.'/css/back.css', [], BIN_EVENT_CALENDAR_VERSION);
		wp_enqueue_script('bin-event-calendar-back', BIN_EVENT_CALENDAR_URL.'/js/back.js', [], BIN_EVENT_CALENDAR_VERSION, true);
	}
	
	/**
	 * Add frontend style/script
	 */
	function enqueue_scripts()
	{
		wp_register_style('bin-event-calendar-front', BIN_EVENT_CALENDAR_URL.'/css/front.css', false, BIN_EVENT_CALENDAR_VERSION);
	}
	
	/**
	 * Add dependency check message
	 */
	function admin_notice()
	{
		$checkPlugins = [
			'bin-event-calendar' => [
				'name' => 'Bin Contact',
				'path' => 'bin-event-calendar/bin-event-calendar.php',
				'text_activate' => __('The plugin <strong>%s</strong> is recommended to store your contacts in an effective manner.', 'bin-event-calendar')
			]
		];

		$installedPlugins = get_plugins();

		$sitewidePlugins = [];
		if(is_array(get_site_option('active_sitewide_plugins'))) { $sitewidePlugins = get_site_option('active_sitewide_plugins'); }
		$plugins = array_merge(array_flip(get_option('active_plugins')), $sitewidePlugins);

		$message = '';
		foreach($checkPlugins as $checkPluginName => $checkPluginInfo)
		{
			if(!array_key_exists($checkPluginInfo['path'], $plugins))
			{
				if(isset($installedPlugins[$checkPluginInfo['path']]))
				{
					$message .= '<p>'.sprintf($checkPluginInfo['text_activate'], $checkPluginInfo['name']).' <a class="button" href="'.self_admin_url('plugins.php').'">'.__('Activate', 'bin-event-calendar').'</a></p>';
				}
				else
				{
					$url = add_query_arg(['action' => 'install-plugin', 'plugin' => $checkPluginName], self_admin_url('update.php'));
					$message .= '<p>'.sprintf($checkPluginInfo['text_activate'], $checkPluginInfo['name']).' <a class="button" href="'.wp_nonce_url($url, 'install-plugin_'.$checkPluginName).'">'.__('Install', 'bin-event-calendar').'</a></p>';
				}
			}
		}

		if(!empty($message))
		{
			echo '<div id="bin_event_calendar_admin_notice" class="notice notice-error">';
				echo '<p><strong>'.sprintf(__('%s dependency check failed', 'bin-event-calendar'), 'Bin Opt-in').'</strong></p>';
				echo '<div class="bin_event_calendar_admin_notice_message">'.$message.'</div>';
				echo '<p><a href="admin.php?page=bin_event_calendar_option_setting" class="bin_event_calendar_admin_notice_dismiss">'.__('Disable notice', 'bin-event-calendar').'</a></p>';
			echo '</div>';
		}
	}
	
	/**
	 * set dependency check disabled
	 */
	function wp_ajax_bin_event_calendar_admin_notice_dismiss()
	{
		update_option('bin_event_calendar_dependency_check', 'disabled');
		wp_die();
	}
	
	/**
	 * Add custom pages
	 */
	public static function init_custompages()
	{
		add_rewrite_rule('^event/ics', 'index.php?bin-event-calendar-custompage=ics', 'top');
	}

	/**
	 * Add custom pages to query vars
	 * @param array $query_vars
	 * @return string
	 */
	public static function query_vars_custompages($query_vars)
	{
		$query_vars[] = 'bin-event-calendar-custompage';
		return $query_vars;
	}

	// load custom page
	public static function parse_request_custompages(&$wp)
	{
		if(isset($wp->query_vars['bin-event-calendar-custompage']) && $wp->query_vars['bin-event-calendar-custompage'] === 'ics')
		{
			require_once BIN_EVENT_CALENDAR_DIR.'/custompages/ics.php';
			exit();
		}
	}
	
	static function formEventDuration()
	{
		return [
			'one_day' => __('Single day', 'bin-event-calendar'),
			'several_days' => __('Several days', 'bin-event-calendar'),
		];
	}
	
	static function parse_list($list)
	{
		$level_1 = [];
		if(!is_array($list)) {
			$level_1 = preg_split('/[\s,]+/', $list, -1, PREG_SPLIT_NO_EMPTY);
		}

		$level_2 = [];
		foreach($level_1 as $item)
		{
			$level_2[] = preg_split('/[\s:]+/', $item, -1, PREG_SPLIT_NO_EMPTY);
		}
		
		return $level_2;
	}
	
	static function the_excerpt_max_charlength($excerpt, $charlength)
	{
		$html = '';
		$charlength++;
		if(mb_strlen($excerpt) > $charlength)
		{
			$subex = mb_substr($excerpt, 0, $charlength - 5);
			$exwords = explode(' ', $subex);
			$excut = - (mb_strlen($exwords[count($exwords) - 1]));
			if($excut < 0)
			{
				$html .= mb_substr($subex, 0, $excut);
			}
			else
			{
				$html .= $subex;
			}
			$html .= '[...]';
		}
		else
		{
			$html .= $excerpt;
		}
		
		return $html;
	}
	
	static function get_events($atts)
	{
		// use global
		global $wpdb;
		
		$atts = shortcode_atts([
			'page' => isset($_GET['event-page']) ? abs((int)$_GET['event-page']) : 1,
			'posts_per_page' => 20,
			//'group' => '',
			'order' => 'ASC',
			'orderby' => 'date',
			'post_status' => 'publish'
		], $atts);
		
		$orderBy = '';
		if($atts['orderby'] === 'ID') { $orderBy = 'ORDER BY event.ID '.$atts['order']; }
		else if($atts['orderby'] === 'author') { $orderBy = 'ORDER BY event.post_author '.$atts['order']; }
		else if($atts['orderby'] === 'title') { $orderBy = 'ORDER BY event.post_title '.$atts['order']; }
		else if($atts['orderby'] === 'date') { $orderBy = 'ORDER BY IF(duration = \'one_day\', one_date, serval_date_from) '.$atts['order']; }
		else if($atts['orderby'] === 'modified') { $orderBy = 'ORDER BY event.post_modified '.$atts['order']; }
		else if($atts['orderby'] === 'parent') { $orderBy = 'ORDER BY event.post_parent '.$atts['order']; }
		else if($atts['orderby'] === 'rand') { $orderBy = 'ORDER BY RAND()'; }
		else if($atts['orderby'] === 'comment_count') { $orderBy = 'ORDER BY event.comment_count '.$atts['order']; }
		else if($atts['orderby'] === 'menu_order') { $orderBy = 'ORDER BY event.menu_order '.$atts['order']; }
		
		
		// prepare query
		$query = 'SELECT
			event.*,
			duration.meta_value as duration,
			one_date.meta_value as one_date,
			serval_date_from.meta_value as serval_date_from
		FROM
			'.$wpdb->posts.' event
			INNER JOIN '.$wpdb->postmeta.' duration ON duration.post_id = event.ID AND (duration.meta_key = \'bin_event_calendar_event_duration\')
			INNER JOIN '.$wpdb->postmeta.' one_date ON one_date.post_id = event.ID AND (one_date.meta_key = \'bin_event_calendar_event_one_date\')
			INNER JOIN '.$wpdb->postmeta.' serval_date_from ON serval_date_from.post_id = event.ID AND (serval_date_from.meta_key = \'bin_event_calendar_event_serval_date_from\')
		WHERE
				event.post_type IN (\'event\')
			AND
				event.post_status = \''.$atts['post_status'].'\' '.$orderBy;
		
		$total_query = 'SELECT COUNT(1) FROM ('.$query.') AS combined_table';
		$total = $wpdb->get_var($total_query);
		
		// Construct SQL query and get results
		$result = [];
		$result['posts'] = $wpdb->get_results($query.' LIMIT '.(($atts['page'] * $atts['posts_per_page']) - $atts['posts_per_page']).', '.$atts['posts_per_page'], OBJECT);
		$result['max_num_pages'] = ceil($total/$atts['posts_per_page']);
		
		return $result;
	}
	
	static function get_events_upcoming($atts)
	{
		// use global
		global $wpdb;
		
		$atts = shortcode_atts([
			'page' => isset($_GET['event-page']) ? abs((int)$_GET['event-page']) : 1,
			'posts_per_page' => 20,
			//'group' => '',
			'order' => 'ASC',
			'orderby' => 'date',
			'post_status' => 'publish'
		], $atts);
		
		$orderBy = '';
		if($atts['orderby'] === 'ID') { $orderBy = 'ORDER BY event.ID '.$atts['order']; }
		else if($atts['orderby'] === 'author') { $orderBy = 'ORDER BY event.post_author '.$atts['order']; }
		else if($atts['orderby'] === 'title') { $orderBy = 'ORDER BY event.post_title '.$atts['order']; }
		else if($atts['orderby'] === 'date') { $orderBy = 'ORDER BY IF(duration = \'one_day\', one_date, serval_date_from) '.$atts['order']; }
		else if($atts['orderby'] === 'modified') { $orderBy = 'ORDER BY event.post_modified '.$atts['order']; }
		else if($atts['orderby'] === 'parent') { $orderBy = 'ORDER BY event.post_parent '.$atts['order']; }
		else if($atts['orderby'] === 'rand') { $orderBy = 'ORDER BY RAND()'; }
		else if($atts['orderby'] === 'comment_count') { $orderBy = 'ORDER BY event.comment_count '.$atts['order']; }
		else if($atts['orderby'] === 'menu_order') { $orderBy = 'ORDER BY event.menu_order '.$atts['order']; }
		
		// prepare query
		$query = 'SELECT
			event.*,
			duration.meta_value as duration,
			one_date.meta_value as one_date,
			serval_date_from.meta_value as serval_date_from,
			serval_date_to.meta_value as serval_date_to
		FROM
			'.$wpdb->posts.' event
			INNER JOIN '.$wpdb->postmeta.' duration ON duration.post_id = event.ID AND (duration.meta_key = \'bin_event_calendar_event_duration\')
			INNER JOIN '.$wpdb->postmeta.' one_date ON one_date.post_id = event.ID AND (one_date.meta_key = \'bin_event_calendar_event_one_date\')
			INNER JOIN '.$wpdb->postmeta.' serval_date_from ON serval_date_from.post_id = event.ID AND (serval_date_from.meta_key = \'bin_event_calendar_event_serval_date_from\')
			INNER JOIN '.$wpdb->postmeta.' serval_date_to ON serval_date_to.post_id = event.ID AND (serval_date_to.meta_key = \'bin_event_calendar_event_serval_date_to\')
		WHERE
			(
				event.post_type IN (\'event\') AND event.post_status = \''.$atts['post_status'].'\'
			)
			AND
			(
				(
					duration.meta_value = \'one_day\' AND one_date.meta_value >= DATE(NOW())
				)
				OR
				(
					duration.meta_value = \'several_days\' AND serval_date_to.meta_value >= DATE(NOW())
				)
			) '.$orderBy;
		
		$total_query = 'SELECT COUNT(1) FROM ('.$query.') AS combined_table';
		$total = $wpdb->get_var($total_query);
		
		// Construct SQL query and get results
		$result = [];
		$result['posts'] = $wpdb->get_results($query.' LIMIT '.(($atts['page'] * $atts['posts_per_page']) - $atts['posts_per_page']).', '.$atts['posts_per_page'], OBJECT);
		$result['max_num_pages'] = ceil($total/$atts['posts_per_page']);
		
		return $result;
	}
	
	static function formatEventDate($event_id, $format='l, j. F Y')
	{
		$date_string = '';
		$event_meta = get_post_meta($event_id);

		switch($event_meta['bin_event_calendar_event_duration'][0])
		{
			case 'one_day':
				$date_string .= date_i18n($format, strtotime($event_meta['bin_event_calendar_event_one_date'][0]));
				if($event_meta['bin_event_calendar_event_one_date_time_from'][0]) { $date_string .= ' - '.$event_meta['bin_event_calendar_event_one_date_time_from'][0]; }
				if($event_meta['bin_event_calendar_event_one_date_time_to'][0]) { $date_string .= ' '.__('to', 'bin-event-calendar').' '.$event_meta['bin_event_calendar_event_one_date_time_to'][0]; }
			break;
			
			case 'several_days':
				if($event_meta['bin_event_calendar_event_serval_date_from'][0])
				{
					$date_string .= date_i18n($format, strtotime($event_meta['bin_event_calendar_event_serval_date_from'][0]));
					if($event_meta['bin_event_calendar_event_serval_date_from_time_from'][0]) { $date_string .= ', '.$event_meta['bin_event_calendar_event_serval_date_from_time_from'][0]; }
				}
				if($event_meta['bin_event_calendar_event_serval_date_to'][0])
				{
					$date_string .= ' - '.date_i18n($format, strtotime($event_meta['bin_event_calendar_event_serval_date_to'][0]));
					if($event_meta['bin_event_calendar_event_serval_date_to_time_to'][0]) { $date_string .= ', '.$event_meta['bin_event_calendar_event_serval_date_to_time_to'][0]; }
				}
			break;
		}
		
		return $date_string;
	}
	
	static function ical($events, $saveAttachmentToTemp=false)
	{
		$vCalendar = new \Eluceo\iCal\Component\Calendar(get_site_url());
		if(isset($events))
		{
			// accepts one single event or array of multiple events
			if(!is_array($events)) { $myEvents[] = $events; }
			else { $myEvents = $events; }
			
			foreach($myEvents as $event)
			{
				$vEvent = new \Eluceo\iCal\Component\Event();
				$vEvent->setSummary($event->post_title);
				
				$event_meta = get_post_meta($event->ID);
				switch($event_meta['bin_event_calendar_event_duration'][0])
				{
					case 'one_day':
						if($event_meta['bin_event_calendar_event_one_date'][0])
						{
							if($event_meta['bin_event_calendar_event_one_date_time_from'][0] && $event_meta['bin_event_calendar_event_one_date_time_to'][0])
							{
								$vEvent->setDtStart(new \DateTime($event_meta['bin_event_calendar_event_one_date'][0].' '.$event_meta['bin_event_calendar_event_one_date_time_from'][0].':00'));
								$vEvent->setDtEnd(new \DateTime($event_meta['bin_event_calendar_event_one_date'][0].' '.$event_meta['bin_event_calendar_event_one_date_time_to'][0].':00'));
							}
							else
							{
								$vEvent->setDtStart(new \DateTime($event_meta['bin_event_calendar_event_one_date'][0]));
								$vEvent->setNoTime(true);
							}
						}
					break;

					case 'several_days':
						if($event_meta['bin_event_calendar_event_serval_date_from'][0] && $event_meta['bin_event_calendar_event_serval_date_to'][0])
						{
							if($event_meta['bin_event_calendar_event_serval_date_from_time_from'][0] && $event_meta['bin_event_calendar_event_serval_date_to_time_to'][0])
							{
								$vEvent->setDtStart(new \DateTime($event_meta['bin_event_calendar_event_serval_date_from'][0].' '.$event_meta['bin_event_calendar_event_serval_date_from_time_from'][0].':00'));
								$vEvent->setDtEnd(new \DateTime($event_meta['bin_event_calendar_event_serval_date_to'][0].' '.$event_meta['bin_event_calendar_event_serval_date_to_time_to'][0].':00'));
							}
							else
							{
								$vEvent->setDtStart(new \DateTime($event_meta['bin_event_calendar_event_serval_date_from'][0]));
								$vEvent->setDtEnd(new \DateTime($event_meta['bin_event_calendar_event_serval_date_to'][0]));
								$vEvent->setNoTime(true);
							}
						}
					break;
				}

				$vCalendar->addComponent($vEvent);
			}
		}
		
		if($saveAttachmentToTemp === true)
		{
			$icalDir = get_temp_dir().'events.ics';
			file_put_contents($icalDir, $vCalendar->render());
			return $icalDir;
		}
		
		return $vCalendar->render();
	}
	
	/*
	 * default options
	 */
	private static function options()
	{
		return [];
	}
	
	/**
	 * fires on activate plugin
	 * add initial options to settings page
	 * register schedule events
	 */
	public static function activate()
	{
		global $wp_rewrite;
		self::init_custompages();
		$wp_rewrite->flush_rules(false);
		
		foreach(self::options() as $option => $value)
		{
			if(!get_option($option))
			{
				add_option($option, $value);
			}
		}
    }
	
	/**
	 * fires on deactivate plugin
	 * unregister schedule events
	 */
	public static function deactivate()
	{
		self::uninstall();
	}
	
	/**
	 * fires on uninstall plugin
	 * delete options
	 */
	public static function uninstall()
	{
		foreach(self::options() as $option => $value)
		{
			delete_option($option);
		}
	}
}