<?php

/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

class BinEventCalendar_Dashboard
{
	function __construct()
	{
		if(current_user_can('editor') || current_user_can('administrator'))
		{
			add_action('wp_dashboard_setup', [$this, 'add_dashboard_widgets']);
			add_action('wp_ajax_bin_event_calendar_widget', [$this, 'wp_ajax_bin_event_calendar_widget']);
		}
	}
	
	function add_dashboard_widgets()
	{
		wp_add_dashboard_widget('bin_event_calendar_widget', __('Calendar', 'bin-event-calendar'), [$this, 'bin_event_calendar_widget']);
	}
	
	function bin_event_calendar_widget($post, $callback_args)
	{
		echo '<div class="bin_event_calendar_widget_box bin_event_calendar_widget_box_blue">';
			echo '<div class="inner">';
				echo '<div id="bin_event_calendar_widget_table">';
					echo self::prepare_calendar();
				echo '</div>';
			echo '</div>';
		echo '</div>';
	}
	
	function wp_ajax_bin_event_calendar_widget()
	{
		echo self::prepare_calendar();
		wp_die();
	}
	
	static function prepare_calendar()
	{
		// use global
		global $wpdb;
		
		$month = (isset($_REQUEST['month']) && is_numeric($_REQUEST['month'])) ? $_REQUEST['month'] : date('m');
		$year = (isset($_REQUEST['year']) && is_numeric($_REQUEST['year'])) ? $_REQUEST['year'] : date('Y');
		
		$dateObj = DateTime::createFromFormat('!m', $month);
		$monthName = date_i18n('F', $dateObj->getTimestamp());
		
		$days = cal_days_in_month(CAL_GREGORIAN, $month, $year);
		$firstWeekday = date('N', mktime(0, 0, 0, $month, 1, $year));
		
		$navigation[] = '';
		$navigation['currentMonth'] = date('m');
		$navigation['currentYear'] = date('Y');

		if($month+1 <= 12)
		{
			$navigation['nextMonth'] = sprintf('%02d', $month+1);
			$navigation['nextYear'] = $year;
		}
		else
		{
			$navigation['nextMonth'] = '01';
			$navigation['nextYear'] = $year+1;
		}

		if($month-1 >= 1)
		{
			$navigation['prevMonth'] = sprintf('%02d', $month-1);
			$navigation['prevYear'] = $year;
		}
		else
		{
			$navigation['prevMonth'] = '12';
			$navigation['prevYear'] = $year-1;
		}
		
		// get events of month
		$events_search_query = "SELECT SQL_CALC_FOUND_ROWS
			event.*,
			duration.meta_value as duration,
			one_date.meta_value as one_date,
			serval_date_from.meta_value as serval_date_from,
			serval_date_to.meta_value as serval_date_to
		FROM
			$wpdb->posts event
			INNER JOIN $wpdb->postmeta duration ON duration.post_id = event.ID AND (duration.meta_key = 'bin_event_calendar_event_duration')
			INNER JOIN $wpdb->postmeta one_date ON one_date.post_id = event.ID AND (one_date.meta_key = 'bin_event_calendar_event_one_date')
			INNER JOIN $wpdb->postmeta serval_date_from ON serval_date_from.post_id = event.ID AND (serval_date_from.meta_key = 'bin_event_calendar_event_serval_date_from')
			INNER JOIN $wpdb->postmeta serval_date_to ON serval_date_to.post_id = event.ID AND (serval_date_to.meta_key = 'bin_event_calendar_event_serval_date_to')
		WHERE (
			event.post_type IN ('event') AND event.post_status = 'publish'
		)
		AND
		(
			(
				duration.meta_value = 'one_day' AND (
					one_date.meta_value >= '$year-$month-01' AND one_date.meta_value <= '$year-$month-31'
				)
			)
			OR
			(
				duration.meta_value = 'several_days' AND (
					('$year-$month' >= DATE_FORMAT(STR_TO_DATE(serval_date_from.meta_value, '%Y-%c-%e'),'%Y-%m'))
						OR
					('$year-$month' <= DATE_FORMAT(STR_TO_DATE(serval_date_to.meta_value, '%Y-%c-%e'),'%Y-%m'))
				)
			)
		)";
		$events = $wpdb->get_results($events_search_query, OBJECT);
		
		$calendar = '';
		$calendar .= '<div class="bin_event_calendar_container">';
			$calendar .= '<div class="bin_event_calendar_date">';
				$calendar .= '<p>'.$monthName.' '.$year.'</p>';
			$calendar .= '</div>';
			$calendar .= '<div class="bin_event_calendar_buttons">';	
				$calendar .= '<a href="#" data-target="#bin_event_calendar_widget_table" data-attributes=\'{ "action": "bin_event_calendar_widget", "year": "'.$navigation['currentYear'].'", "month": "'.$navigation['currentMonth'].'" }\' class="bin_event_calendar_ajax button">'.__('Today', 'bin-event-calendar').'</a>';
				$calendar .= '<a href="#" data-target="#bin_event_calendar_widget_table" data-attributes=\'{ "action": "bin_event_calendar_widget", "year": "'.$navigation['prevYear'].'", "month": "'.$navigation['prevMonth'].'" }\' class="bin_event_calendar_ajax button">'.__('Back', 'bin-event-calendar').'</a>';
				$calendar .= '<a href="#" data-target="#bin_event_calendar_widget_table" data-attributes=\'{ "action": "bin_event_calendar_widget", "year": "'.$navigation['nextYear'].'", "month": "'.$navigation['nextMonth'].'" }\' class="bin_event_calendar_ajax button">'.__('Next', 'bin-event-calendar').'</a>';
			$calendar .= '</div>';
		$calendar .= '</div>';
		$calendar .= '<table class="bin_event_calendar_table">';
			$calendar .= '<tr>';
				$calendar .= '<th>'.__('Monday').'</th>';
				$calendar .= '<th>'.__('Tuesday').'</th>';
				$calendar .= '<th>'.__('Wednesday').'</th>';
				$calendar .= '<th>'.__('Thursday').'</th>';
				$calendar .= '<th>'.__('Friday').'</th>';
				$calendar .= '<th>'.__('Saturday').'</th>';
				$calendar .= '<th>'.__('Sunday').'</th>';
			$calendar .= '</tr>';

			$day = 1;
			for($weekCount=1; $weekCount<=ceil(($days+$firstWeekday)/7); $weekCount++)
			{
				$calendar .= '<tr>';
					for($weekDayCount=1; $weekDayCount<=7; $weekDayCount++)
					{
						if($weekCount===1 && $weekDayCount<=$firstWeekday)
						{
							$calendar .= '<td>&nbsp;</td>';
						}
						else
						{
							$calendar .= self::prepare_calender_items($year, $month, $day, $days, $events);
							$day++;
						}
					}
				$calendar .= '</tr>';
			}

		$calendar .= '</table>';
				
		return $calendar;
	}
	
	static function prepare_calender_items($year, $month, $day, $days, $events)
	{	
		// calculate days remaining
		$end_date = get_option('bin_event_calendar_campaign_end_date');
		$date_diff = strtotime($end_date) - strtotime($year.'-'.$month.'-'.sprintf('%02d', $day));
		$date_days_remaining = floor($date_diff/60/60/24);
		
		// search for events
		$labels = '';
		$labelsTooltips = '<div class="bin_event_calendar_tooltip_text">';
		
			// events
			if($events)
			{
				foreach($events as $event)
				{
					if(
						(($event->duration === 'one_day') && ($event->one_date === $year.'-'.$month.'-'.sprintf('%02d', $day)))
							OR
						(($event->duration === 'several_days' && ($year.'-'.$month.'-'.sprintf('%02d', $day) >= $event->serval_date_from AND $year.'-'.$month.'-'.sprintf('%02d', $day) <= $event->serval_date_to)))
					)
					{
						$labels .= '<span class="bin_event_calendar_label bin_event_calendar_label_event" title="'.$event->post_title.'">'.$event->post_title.'</span>';
						$labelsTooltips .= '<p><b>'.$event->post_title.'</b><br>'.BinEventCalendar::formatEventDate($event->ID).'</p><a href="post.php?post='.$event->ID.'&action=edit" class="button">'.__('Edit', 'bin-event-calendar').'</a>';
					}
				}
			}
			
		$labelsTooltips .= '</div>';
		
		// default tooltips
		$defaultTooltips = '<div class="bin_event_calendar_tooltip_text">';
			$defaultTooltips .= '<p><b>'.__('No events', 'bin-event-calendar').'</b></p>';
			$defaultTooltips .= '<a href="post-new.php?post_type=event" class="button">'.__('Add event', 'bin-event-calendar').'</a>';
		$defaultTooltips .= '</div>';
		
		// add classes
		$class = 'bin_event_calendar_tooltip';
		if($year.'-'.$month.'-'.sprintf('%02d', $day) === date('Y-m-d')) { $class .= ' bin_event_calendar_today'; }
		if($year.'-'.$month.'-'.sprintf('%02d', $day) === $end_date) { $class .= ' bin_event_calendar_end_date'; }
		
		// events
		return '<td class="'.trim($class).'">'.(($day <= $days) ? $day : '').($labels ? $labels.$labelsTooltips : $defaultTooltips).'</td>';
	}
}