<?php

class BinEventCalendar_Shortcode
{
	public function __construct()
	{
		add_shortcode('event_list', [$this, 'event_list_shortcode']);
		add_shortcode('event_grid', [$this, 'event_grid_shortcode']);
	}
	
	public static function get_event_title($event, $atts)
	{
		$html = '';
		$html .= '<div class="bin_event_calendar_title">';
			$html .= '<h3>';
				if(!empty($event->post_content)) { $html .= '<a href="'.get_permalink($event->ID).'">'; }
					$html .= $event->post_title;
				if(!empty($event->post_content)) { $html .= '</a>'; }
			$html .= '</h3>';
		$html .= '</div>';	
		return $html;
	}
	
	public static function remove_empty_categories($event, $metaItems, $postmeta)
	{
		foreach($metaItems as $key => $value)
		{
			switch($value[0])
			{
				case 'categories': {
					$terms = wp_get_post_terms($event->ID, 'event_category');
					if(!$terms) { unset($metaItems[$key]); }
				} break;
			}
		}
		
		return $metaItems;
	}
	
	public static function remove_empty_place($event, $metaItems, $postmeta)
	{
		foreach($metaItems as $key => $value)
		{
			switch($value[0])
			{
				case 'place': {
					if(empty($postmeta['bin_event_calendar_event_place'][0])) { unset($metaItems[$key]); }
				}
			}
		}
		
		return $metaItems;
	}
	
	public static function get_event_meta($event, $atts, $postmeta, $metaSet = 'meta_items')
	{
		$html = '';
		$metaItems = BinEventCalendar::parse_list($atts[$metaSet]);
		if($atts['show_empty_categories'] === 'no') { $metaItems = self::remove_empty_categories($event, $metaItems, $postmeta); }
		if($atts['show_empty_place'] === 'no') { $metaItems = self::remove_empty_place($event, $metaItems, $postmeta); }
		
		$item_len = count($metaItems);
		if($item_len > 0)
		{
			$html .= '<div class="bin_event_calendar_meta">';
				$item_i = 0;
				foreach($metaItems as $metaItem)
				{
					$html .= '<span class="bin_event_calendar_meta_item bin_event_calendar_meta_'.$metaItem[0].'">';
						switch($metaItem[0])
						{
							case 'date': {
								$html .= '<span class="bin_event_calendar_meta_item_inner">';
									$html .= '<span class="bin_event_calendar_meta_icon">'.$atts['date_icon'].'</span>';
									$html .= '<span class="bin_event_calendar_meta_text">'.(isset($metaItem[1]) ? $metaItem[1] : BinEventCalendar::formatEventDate($event->ID, $atts['date_format'])).'</span>';
								$html .= '</span>';
							} break;
							case 'author': {
								$html .= '<a href="'.get_author_posts_url($event->post_author).'" class="bin_event_calendar_meta_item_inner">';
									$html .= '<span class="bin_event_calendar_meta_icon">'.$atts['author_icon'].'</span>';
									$html .= '<span class="bin_event_calendar_meta_text">'.(isset($metaItem[1]) ? $metaItem[1] : get_the_author()).'</span>';
								$html .= '</a>';
							} break;
							case 'ical': {
								$html .= '<a href="'.get_site_url().'/event/ics?id='.$event->ID.'" class="bin_event_calendar_meta_item_inner">';
									$html .= '<span class="bin_event_calendar_meta_icon">'.$atts['ical_icon'].'</span>';
									$html .= '<span class="bin_event_calendar_meta_text">'.(isset($metaItem[1]) ? $metaItem[1] : __('Copy to calendar', 'bin-event-calendar')).'</span>';
								$html .= '</a>';
							} break;
							case 'place': {
								$html .= '<span class="bin_event_calendar_meta_item_inner">';
									$html .= '<span class="bin_event_calendar_meta_icon">'.$atts['place_icon'].'</span>';
									$html .= '<span class="bin_event_calendar_meta_text">'.(!empty($postmeta['bin_event_calendar_event_place'][0]) ? $postmeta['bin_event_calendar_event_place'][0] : __('No location', 'bin-event-calendar')).'</span>';
								$html .= '</span>';
							} break;
							case 'categories': {
								$terms = wp_get_post_terms($event->ID, 'event_category');
								if($terms)
								{
									$html .= '<span class="bin_event_calendar_meta_terms">';
										foreach($terms as $term)
										{
											$html .= '<a class="bin_event_calendar_meta_term bin_event_calendar_meta_term_'.$term->ID.'" href="'.get_permalink($term->ID).'">';
												$html .= '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="circle" class="svg-inline--fa fa-circle fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8z"></path></svg>';
												$html .= '<span>'.$term->name.'</span>';
											$html .= '</a>';
										}
									$html .= '</span>';
								}
								else
								{
									$html .= '<span class="bin_event_calendar_meta_terms">';
										$html .= '<span class="bin_event_calendar_meta_term bin_event_calendar_meta_term_uncategorized">';
											$html .= '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="circle" class="svg-inline--fa fa-circle fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8z"></path></svg>';
											$html .= '<span>'.(isset($metaItem[1]) ? $metaItem[1] : __('Uncategorized', 'bin-event-calendar')).'</span>';
										$html .= '</span>';
									$html .= '</span>';
								}
							} break;
						}
					$html .= '</span>';

					if($item_i++ !== $item_len-1)
					{
						$html .= '<span class="bin_event_calendar_meta_item bin_event_calendar_meta_item_sep">';
							$html .= '<span class="bin_event_calendar_meta_item_sep_icon">';
								$html .= $atts['seperator_icon'];
							$html .= '</span>';
						$html .= '</span>';
					}
				}
			$html .= '</div>';
		}
		
		return $html;
	}
	
	public static function event_list_shortcode($atts)
	{
		wp_enqueue_style('bin-event-calendar-front');
		
		$atts = shortcode_atts([
			'display_mode' => 'all', // all or upcoming
			'show_first_seperator' => 'yes', // yes or no
			'show_last_seperator' => 'yes', // yes or no
			'show_between_seperator' => 'yes', // yes or no
			'title_position' => 'inside', // before, inside
			'meta_position' => 'inside', // before, inside, end
			'meta_items' => 'date,place,ical', // date,author,ical,place,categories (optional change text e.g. ical:Calendar)
			'meta2_position' => 'end', // before, inside, end
			'meta2_items' => 'categories', // date,author,ical,place,categories (optional change text e.g. ical:Calendar)
			'show_empty_categories' => 'no',
			'show_empty_place' => 'no',
			'date_format' => 'D, j. M Y',
			'date_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="calendar-alt" class="svg-inline--fa fa-calendar-alt fa-w-14" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path fill="currentColor" d="M0 464c0 26.5 21.5 48 48 48h352c26.5 0 48-21.5 48-48V192H0v272zm320-196c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zm0 128c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zM192 268c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zm0 128c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zM64 268c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12H76c-6.6 0-12-5.4-12-12v-40zm0 128c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12H76c-6.6 0-12-5.4-12-12v-40zM400 64h-48V16c0-8.8-7.2-16-16-16h-32c-8.8 0-16 7.2-16 16v48H160V16c0-8.8-7.2-16-16-16h-32c-8.8 0-16 7.2-16 16v48H48C21.5 64 0 85.5 0 112v48h448v-48c0-26.5-21.5-48-48-48z"></path></svg>',
			'author_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="user" class="svg-inline--fa fa-user fa-w-14" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path fill="currentColor" d="M224 256c70.7 0 128-57.3 128-128S294.7 0 224 0 96 57.3 96 128s57.3 128 128 128zm89.6 32h-16.7c-22.2 10.2-46.9 16-72.9 16s-50.6-5.8-72.9-16h-16.7C60.2 288 0 348.2 0 422.4V464c0 26.5 21.5 48 48 48h352c26.5 0 48-21.5 48-48v-41.6c0-74.2-60.2-134.4-134.4-134.4z"></path></svg>',
			'ical_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="share" class="svg-inline--fa fa-share fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M503.691 189.836L327.687 37.851C312.281 24.546 288 35.347 288 56.015v80.053C127.371 137.907 0 170.1 0 322.326c0 61.441 39.581 122.309 83.333 154.132 13.653 9.931 33.111-2.533 28.077-18.631C66.066 312.814 132.917 274.316 288 272.085V360c0 20.7 24.3 31.453 39.687 18.164l176.004-152c11.071-9.562 11.086-26.753 0-36.328z"></path></svg>',
			'place_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="far" data-icon="map" class="svg-inline--fa fa-map fa-w-18" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 576 512"><path fill="currentColor" d="M560.02 32c-1.96 0-3.98.37-5.96 1.16L384.01 96H384L212 35.28A64.252 64.252 0 0 0 191.76 32c-6.69 0-13.37 1.05-19.81 3.14L20.12 87.95A32.006 32.006 0 0 0 0 117.66v346.32C0 473.17 7.53 480 15.99 480c1.96 0 3.97-.37 5.96-1.16L192 416l172 60.71a63.98 63.98 0 0 0 40.05.15l151.83-52.81A31.996 31.996 0 0 0 576 394.34V48.02c0-9.19-7.53-16.02-15.98-16.02zM224 90.42l128 45.19v285.97l-128-45.19V90.42zM48 418.05V129.07l128-44.53v286.2l-.64.23L48 418.05zm480-35.13l-128 44.53V141.26l.64-.24L528 93.95v288.97z"></path></svg>',
			'seperator_icon' => '<svg width="100%" height="100%" viewBox="0 0 24 384" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xml:space="preserve" xmlns:serif="http://www.serif.com/" style="fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;"><path d="M24,372l0,-360c0,-6.6 -2.7,-12 -6,-12l-12,0c-3.3,0 -6,5.4 -6,12l0,360c0,6.6 2.7,12 6,12l12,0c3.3,0 6,-5.4 6,-12Z"/></svg>',
			'show_image' => 'yes',
			'image_size' => 'large',
			'image_fallback' => BIN_EVENT_CALENDAR_URL.'/images/no-event.jpg',
			'show_excerpt' => 'yes',
			'excerpt_charlength' => 420,
			'show_button' => 'no',
			'button_target' => '_self',
			'button_text' => __('Read more', 'bin-contact'),
			'show_button_ical' => 'no',
			'button_ical_target' => '_blank',
			'button_ical_text' => __('Copy to calendar', 'bin-contact'),
			'show_pagination' => 'yes',
			'pagination_mid_size' => 3,
			'pagination_prev_text' => __('Back', 'bin-contact'),
			'pagination_next_text' => __('Next', 'bin-contact'),
			'pagination_screen_reader_text' => __('Posts navigation', 'bin-contact')
		], $atts);
		
		$events = '';
		if($atts['display_mode'] === 'all')
		{
			$events = BinEventCalendar::get_events($atts);
		}
		else if($atts['display_mode'] === 'upcoming')
		{
			$events = BinEventCalendar::get_events_upcoming($atts);
		}

		$html = '';
		$events_len = count($events['posts']);
		if($events_len > 0)
		{
			if($atts['show_first_seperator'] === 'yes')
			{
				$html .= '<div class="bin_event_calendar_sep bin_event_calendar_sep_first"><div class="bin_event_calendar_sep_inner"><hr></div></div>';
			}
			
			$html .= '<div class="bin_event_list">';
			
				$events_i = 1;
				foreach($events['posts'] as $event)
				{
					$postmeta = get_post_meta($event->ID);
					setup_postdata($event);

					$html .= '<div class="bin_event_calendar">';

						if($atts['title_position'] === 'before')
						{
							$html .= '<div class="bin_event_calendar_title_before">';
								$html .= self::get_event_title($event, $atts);
							$html .= '</div>';
						}
						if($atts['meta_position'] === 'before')
						{
							$html .= '<div class="bin_event_calendar_meta_before">';
								$html .= self::get_event_meta($event, $atts, $postmeta, 'meta_items');
							$html .= '</div>';
						}
						if($atts['meta2_position'] === 'before')
						{
							$html .= '<div class="bin_event_calendar_meta2_before">';
								$html .= self::get_event_meta($event, $atts, $postmeta, 'meta2_items');
							$html .= '</div>';
						}

						$html .= '<div class="bin_event_calendar_main">';

							if($atts['show_image'] === 'yes')
							{

								$html .= '<div class="bin_event_calendar_main_image">';
									if(!empty($event->post_content)) { $html .= '<a href="'.get_permalink($event->ID).'">'; }
										$postUrl = get_the_post_thumbnail_url($event->ID, $atts['image_size']);
										if($postUrl) { $html .= '<img src="'.$postUrl.'">'; }
										else { $html .= '<img src="'.$atts['image_fallback'].'">'; }
									if(!empty($event->post_content)) { $html .= '</a>'; }
								$html .= '</div>';
							}

							$html .= '<div class="bin_event_calendar_main_text">';
								if($atts['title_position'] === 'inside')
								{
									$html .= '<div class="bin_event_calendar_title_inside">';
										$html .= self::get_event_title($event, $atts);
									$html .= '</div>';
								}
								if($atts['meta_position'] === 'inside')
								{
									$html .= '<div class="bin_event_calendar_meta_inside">';
										$html .= self::get_event_meta($event, $atts, $postmeta, 'meta_items');
									$html .= '</div>';
								}
								if($atts['meta2_position'] === 'inside')
								{
									$html .= '<div class="bin_event_calendar_meta2_inside">';
										$html .= self::get_event_meta($event, $atts, $postmeta, 'meta2_items');
									$html .= '</div>';
								}
								if($atts['show_excerpt'] === 'yes')
								{
									$html .= '<div class="bin_event_calendar_main_text_excerpt">';
										if(!empty(get_the_excerpt($event)))
										{
											$html .= '<p>'.BinEventCalendar::the_excerpt_max_charlength(get_the_excerpt($event), $atts['excerpt_charlength']).'</p>';
										}
										else
										{
											$html .= '<p>'.wp_trim_words($event->post_content, $atts['excerpt_charlength']).'</p>';
										}
									$html .= '</div>';
								}

								if((!empty($event->post_content) && $atts['show_button'] === 'yes') || $atts['show_button_ical'] === 'yes')
								{
									$html .= '<div class="bin_event_calendar_main_text_buttons">';
										if(!empty($event->post_content) && $atts['show_button'] === 'yes')
										{
											$html .= '<a href="'.get_permalink($event->ID).'" target="'.$atts['button_target'].'" class="bin_event_calendar_button">';
												$html .= '<span class="bin_event_calendar_button_icon"><svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="angle-double-right" class="svg-inline--fa fa-angle-double-right fa-w-14" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path fill="currentColor" d="M224.3 273l-136 136c-9.4 9.4-24.6 9.4-33.9 0l-22.6-22.6c-9.4-9.4-9.4-24.6 0-33.9l96.4-96.4-96.4-96.4c-9.4-9.4-9.4-24.6 0-33.9L54.3 103c9.4-9.4 24.6-9.4 33.9 0l136 136c9.5 9.4 9.5 24.6.1 34zm192-34l-136-136c-9.4-9.4-24.6-9.4-33.9 0l-22.6 22.6c-9.4 9.4-9.4 24.6 0 33.9l96.4 96.4-96.4 96.4c-9.4 9.4-9.4 24.6 0 33.9l22.6 22.6c9.4 9.4 24.6 9.4 33.9 0l136-136c9.4-9.2 9.4-24.4 0-33.8z"></path></svg></span>';
												$html .= '<span class="bin_event_calendar_button_text">'.html_entity_decode($atts['button_text']).'</span>';
											$html .= '</a>';
										}
										if($atts['show_button_ical'] === 'yes')
										{
											$html .= '<a href="'.get_site_url().'/event/ics?id='.$event->ID.'"  target="'.$atts['button_ical_target'].'" class="bin_event_calendar_button">';
												$html .= '<span class="bin_event_calendar_button_icon"><svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="share" class="svg-inline--fa fa-share fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M503.691 189.836L327.687 37.851C312.281 24.546 288 35.347 288 56.015v80.053C127.371 137.907 0 170.1 0 322.326c0 61.441 39.581 122.309 83.333 154.132 13.653 9.931 33.111-2.533 28.077-18.631C66.066 312.814 132.917 274.316 288 272.085V360c0 20.7 24.3 31.453 39.687 18.164l176.004-152c11.071-9.562 11.086-26.753 0-36.328z"></path></svg></span>';
												$html .= '<span class="bin_event_calendar_button_text">'.html_entity_decode($atts['button_ical_text']).'</span>';
											$html .= '</a>';
										}
									$html .= '</div>';
								}
								if($atts['meta_position'] === 'end')
								{
									$html .= '<div class="bin_event_calendar_meta_end">';
										$html .= self::get_event_meta($event, $atts, $postmeta, 'meta_items');
									$html .= '</div>';
								}
								if($atts['meta2_position'] === 'end')
								{
									$html .= '<div class="bin_event_calendar_meta2_end">';
										$html .= self::get_event_meta($event, $atts, $postmeta, 'meta2_items');
									$html .= '</div>';
								}
							$html .= '</div>';
						$html .= '</div>';
					$html .= '</div>';

					if($events_i++ !== $events_len)
					{
						if($atts['show_between_seperator'] === 'yes')
						{
							$html .= '<div class="bin_event_calendar_sep bin_event_calendar_sep_between"><div class="bin_event_calendar_sep_inner"><hr></div></div>';
						}
					}
				}
				
			$html .= '</div>';

			if($atts['show_last_seperator'] === 'yes')
			{
				$html .= '<div class="bin_event_calendar_sep bin_event_calendar_sep_last"><div class="bin_event_calendar_sep_inner"><hr></div></div>';
			}
			
			if($atts['show_pagination'] === 'yes')
			{
				if($events['max_num_pages'] > 1)
				{
					$html .= '<div class="bin_event_calendar_pagination">';
						$html .= paginate_links([
							'type' => 'list',
							'base' => add_query_arg('event-page', '%#%'),
							'format' => '',
							'mid_size' => $atts['pagination_mid_size'],
							'prev_text' => $atts['pagination_prev_text'],
							'next_text' => $atts['pagination_next_text'],
							'screen_reader_text' => $atts['pagination_screen_reader_text'],
							'total' => $events['max_num_pages'],
							'current' => isset($_GET['event-page']) ? abs((int)$_GET['event-page']) : 1
						]);
					$html .= '</div>';
				}
			}
		}
		
		wp_reset_postdata();
		
		if(empty($html)) { return __('No events found...', 'bin-event-calendar'); }
		return $html;
	}
	
	public static function event_grid_shortcode($atts)
	{
		wp_enqueue_style('bin-event-calendar-front');
		
		$atts = shortcode_atts([
			'display_mode' => 'all',
			'items_per_row' => 3, // 1-6
			'seperate_rows' => 'no', // yes or no
			'show_first_seperator' => 'no', // yes or no
			'show_last_seperator' => 'no', // yes or no
			'show_between_seperator' => 'no', // yes or no
			'title_position' => 'after', // before or after
			'meta_position' => 'after', // before, after, end
			'meta_items' => 'categories,place', // date,author,ical,place,categories (optional change text e.g. ical:Calendar)
			'meta2_position' => 'end', // before, after, end
			'meta2_items' => 'date,ical', // date,author,ical,place,categories (optional change text e.g. ical:Calendar)
			'show_empty_categories' => 'no',
			'show_empty_place' => 'no',
			'date_format' => 'D, j. M Y',
			'date_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="calendar-alt" class="svg-inline--fa fa-calendar-alt fa-w-14" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path fill="currentColor" d="M0 464c0 26.5 21.5 48 48 48h352c26.5 0 48-21.5 48-48V192H0v272zm320-196c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zm0 128c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zM192 268c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zm0 128c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12h-40c-6.6 0-12-5.4-12-12v-40zM64 268c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12H76c-6.6 0-12-5.4-12-12v-40zm0 128c0-6.6 5.4-12 12-12h40c6.6 0 12 5.4 12 12v40c0 6.6-5.4 12-12 12H76c-6.6 0-12-5.4-12-12v-40zM400 64h-48V16c0-8.8-7.2-16-16-16h-32c-8.8 0-16 7.2-16 16v48H160V16c0-8.8-7.2-16-16-16h-32c-8.8 0-16 7.2-16 16v48H48C21.5 64 0 85.5 0 112v48h448v-48c0-26.5-21.5-48-48-48z"></path></svg>',
			'author_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="user" class="svg-inline--fa fa-user fa-w-14" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path fill="currentColor" d="M224 256c70.7 0 128-57.3 128-128S294.7 0 224 0 96 57.3 96 128s57.3 128 128 128zm89.6 32h-16.7c-22.2 10.2-46.9 16-72.9 16s-50.6-5.8-72.9-16h-16.7C60.2 288 0 348.2 0 422.4V464c0 26.5 21.5 48 48 48h352c26.5 0 48-21.5 48-48v-41.6c0-74.2-60.2-134.4-134.4-134.4z"></path></svg>',
			'ical_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="share" class="svg-inline--fa fa-share fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M503.691 189.836L327.687 37.851C312.281 24.546 288 35.347 288 56.015v80.053C127.371 137.907 0 170.1 0 322.326c0 61.441 39.581 122.309 83.333 154.132 13.653 9.931 33.111-2.533 28.077-18.631C66.066 312.814 132.917 274.316 288 272.085V360c0 20.7 24.3 31.453 39.687 18.164l176.004-152c11.071-9.562 11.086-26.753 0-36.328z"></path></svg>',
			'place_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="far" data-icon="map" class="svg-inline--fa fa-map fa-w-18" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 576 512"><path fill="currentColor" d="M560.02 32c-1.96 0-3.98.37-5.96 1.16L384.01 96H384L212 35.28A64.252 64.252 0 0 0 191.76 32c-6.69 0-13.37 1.05-19.81 3.14L20.12 87.95A32.006 32.006 0 0 0 0 117.66v346.32C0 473.17 7.53 480 15.99 480c1.96 0 3.97-.37 5.96-1.16L192 416l172 60.71a63.98 63.98 0 0 0 40.05.15l151.83-52.81A31.996 31.996 0 0 0 576 394.34V48.02c0-9.19-7.53-16.02-15.98-16.02zM224 90.42l128 45.19v285.97l-128-45.19V90.42zM48 418.05V129.07l128-44.53v286.2l-.64.23L48 418.05zm480-35.13l-128 44.53V141.26l.64-.24L528 93.95v288.97z"></path></svg>',
			'seperator_icon' => '<svg width="100%" height="100%" viewBox="0 0 24 384" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xml:space="preserve" xmlns:serif="http://www.serif.com/" style="fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;"><path d="M24,372l0,-360c0,-6.6 -2.7,-12 -6,-12l-12,0c-3.3,0 -6,5.4 -6,12l0,360c0,6.6 2.7,12 6,12l12,0c3.3,0 6,-5.4 6,-12Z"/></svg>',
			'show_image' => 'no', // yes or no
			'image_size' => 'large', // or any wp thumbnail size
			'image_fallback' => BIN_EVENT_CALENDAR_URL.'/images/no-event.jpg',
			'show_excerpt' => 'yes', // yes or no
			'excerpt_charlength' => 120,
			'show_button' => 'no', // yes or no
			'button_target' => '_self',
			'button_text' => __('Read more', 'bin-contact'),
			'button_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="angle-double-right" class="svg-inline--fa fa-angle-double-right fa-w-14" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path fill="currentColor" d="M224.3 273l-136 136c-9.4 9.4-24.6 9.4-33.9 0l-22.6-22.6c-9.4-9.4-9.4-24.6 0-33.9l96.4-96.4-96.4-96.4c-9.4-9.4-9.4-24.6 0-33.9L54.3 103c9.4-9.4 24.6-9.4 33.9 0l136 136c9.5 9.4 9.5 24.6.1 34zm192-34l-136-136c-9.4-9.4-24.6-9.4-33.9 0l-22.6 22.6c-9.4 9.4-9.4 24.6 0 33.9l96.4 96.4-96.4 96.4c-9.4 9.4-9.4 24.6 0 33.9l22.6 22.6c9.4 9.4 24.6 9.4 33.9 0l136-136c9.4-9.2 9.4-24.4 0-33.8z"></path></svg>',
			'show_button_ical' => 'no', // yes or no
			'button_ical_target' => '_blank',
			'button_ical_text' => __('Copy to calendar', 'bin-contact'),
			'button_ical_icon' => '<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="share" class="svg-inline--fa fa-share fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M503.691 189.836L327.687 37.851C312.281 24.546 288 35.347 288 56.015v80.053C127.371 137.907 0 170.1 0 322.326c0 61.441 39.581 122.309 83.333 154.132 13.653 9.931 33.111-2.533 28.077-18.631C66.066 312.814 132.917 274.316 288 272.085V360c0 20.7 24.3 31.453 39.687 18.164l176.004-152c11.071-9.562 11.086-26.753 0-36.328z"></path></svg>',
			'show_pagination' => 'yes',
			'pagination_mid_size' => 3,
			'pagination_prev_text' => __('Back', 'bin-contact'),
			'pagination_next_text' => __('Next', 'bin-contact'),
			'pagination_screen_reader_text' => __('Posts navigation', 'bin-contact')
		], $atts);
		
		$events = '';
		if($atts['display_mode'] === 'all')
		{
			$events = BinEventCalendar::get_events($atts);
		}
		else if($atts['display_mode'] === 'upcoming')
		{
			$events = BinEventCalendar::get_events_upcoming($atts);
		}

		$html = '';
		$events_len = count($events['posts']);
		$events_rows_remainer = fmod($events_len, $atts['items_per_row']);
		$events_last_row = ($events_rows_remainer>0) ? $events_rows_remainer : $atts['items_per_row'];
		
		if($events_len > 0)
		{
			if($atts['show_first_seperator'] === 'yes')
			{
				$html .= '<div class="bin_event_calendar_sep bin_event_calendar_sep_first"><div class="bin_event_calendar_sep_inner"><hr></div></div>';
			}
			
			$html .= '<div class="bin_event_grid">';
			
				$events_i = 0;
				foreach($events['posts'] as $event)
				{
					$postmeta = get_post_meta($event->ID);
					setup_postdata($event);
					
					// seperate columns into rows
					if($atts['seperate_rows'] === 'yes')
					{
						if($events_i === 0) { $html .= '<div class="bin_event_row bin_event_row_first">'; }
						else if($events_i % $atts['items_per_row'] === 0) { $html .= '<div class="bin_event_row bin_event_row_between">'; }
					}
					
					$cssClass = 'bin_event_col bin_event_col_'.$atts['items_per_row'];
					if($events_i < $atts['items_per_row']) { $cssClass .= ' bin_event_col_first_row'; }
					if($events_i >= ($events_len - $events_last_row)) { $cssClass .= ' bin_event_col_last_row'; }
					
					$html .= '<div class="'.$cssClass.'">';
						$html .= '<div class="bin_event_calendar">';

							if($atts['title_position'] === 'before')
							{
								$html .= '<div class="bin_event_calendar_title_before">';
									$html .= self::get_event_title($event, $atts);
								$html .= '</div>';
							}
							if($atts['meta_position'] === 'before')
							{
								$html .= '<div class="bin_event_calendar_meta_before">';
									$html .= self::get_event_meta($event, $atts, $postmeta, 'meta_items');
								$html .= '</div>';
							}
							if($atts['meta2_position'] === 'before')
							{
								$html .= '<div class="bin_event_calendar_meta2_before">';
									$html .= self::get_event_meta($event, $atts, $postmeta, 'meta2_items');
								$html .= '</div>';
							}

							$html .= '<div class="bin_event_calendar_main">';

								if($atts['show_image'] === 'yes')
								{
									$html .= '<div class="bin_event_calendar_main_image">';
										if(!empty($event->post_content)) { $html .= '<a href="'.get_permalink($event->ID).'">'; }
											$postUrl = get_the_post_thumbnail_url($event->ID, $atts['image_size']);
											if($postUrl) { $html .= '<img src="'.$postUrl.'">'; }
											else { $html .= '<img src="'.$atts['image_fallback'].'">'; }
										if(!empty($event->post_content)) { $html .= '</a>'; }
									$html .= '</div>';
								}

								$html .= '<div class="bin_event_calendar_main_text">';
									if($atts['title_position'] === 'after')
									{
										$html .= '<div class="bin_event_calendar_title_after">';
											$html .= self::get_event_title($event, $atts);
										$html .= '</div>';
									}
									if($atts['meta_position'] === 'after')
									{
										$html .= '<div class="bin_event_calendar_meta_after">';
											$html .= self::get_event_meta($event, $atts, $postmeta, 'meta_items');
										$html .= '</div>';
									}
									if($atts['meta2_position'] === 'after')
									{
										$html .= '<div class="bin_event_calendar_meta2_after">';
											$html .= self::get_event_meta($event, $atts, $postmeta, 'meta2_items');
										$html .= '</div>';
									}
									if($atts['show_excerpt'] === 'yes')
									{
										$html .= '<div class="bin_event_calendar_main_text_excerpt">';
											if(!empty(get_the_excerpt($event)))
											{
												$html .= '<p>'.BinEventCalendar::the_excerpt_max_charlength(get_the_excerpt($event), $atts['excerpt_charlength']).'</p>';
											}
											else
											{
												$html .= '<p>'.wp_trim_words($event->post_content, $atts['excerpt_charlength']).'</p>';
											}
										$html .= '</div>';
									}

									if((!empty($event->post_content) && $atts['show_button'] === 'yes') || $atts['show_button_ical'] === 'yes')
									{
										$html .= '<div class="bin_event_calendar_main_text_buttons">';
											if(!empty($event->post_content) && $atts['show_button'] === 'yes')
											{
												$html .= '<a href="'.get_permalink($event->ID).'" target="'.$atts['button_target'].'" class="bin_event_calendar_button">';
													$html .= '<span class="bin_event_calendar_button_icon">'.$atts['button_icon'].'</span>';
													$html .= '<span class="bin_event_calendar_button_text">'.html_entity_decode($atts['button_text']).'</span>';
												$html .= '</a>';
											}
											if($atts['show_button_ical'] === 'yes')
											{
												$html .= '<a href="'.get_site_url().'/event/ics?id='.$event->ID.'"  target="'.$atts['button_ical_target'].'" class="bin_event_calendar_button">';
													$html .= '<span class="bin_event_calendar_button_icon">'.$atts['button_ical_icon'].'</span>';
													$html .= '<span class="bin_event_calendar_button_text">'.html_entity_decode($atts['button_ical_text']).'</span>';
												$html .= '</a>';
											}
										$html .= '</div>';
									}
								$html .= '</div>';
								
								if($atts['meta_position'] === 'end')
								{
									$html .= '<div class="bin_event_calendar_meta_end">';
										$html .= self::get_event_meta($event, $atts, $postmeta, 'meta_items');
									$html .= '</div>';
								}
								if($atts['meta2_position'] === 'end')
								{
									$html .= '<div class="bin_event_calendar_meta2_end">';
										$html .= self::get_event_meta($event, $atts, $postmeta, 'meta2_items');
									$html .= '</div>';
								}
								
							$html .= '</div>';
						$html .= '</div>';

					$html .= '</div>';
					
					$events_i++;
					
					// seperate columns into rows
					if($atts['seperate_rows'] === 'yes')
					{
						// end row
						if($events_i !== $events_len)
						{
							if($events_i > 0 && ($events_i % $atts['items_per_row'] === 0))
							{
								$html .= '</div>';
								
								if($atts['show_between_seperator'] === 'yes')
								{
									$html .= '<div class="bin_event_calendar_sep bin_event_calendar_sep_between"><div class="bin_event_calendar_sep_inner"><hr></div></div>';
								}
							}
						}
						else
						{
							$html .= '</div>';
						}
					}
					
					
				}
			
			$html .= '</div>';

			if($atts['show_last_seperator'] === 'yes')
			{
				$html .= '<div class="bin_event_calendar_sep bin_event_calendar_sep_last"><div class="bin_event_calendar_sep_inner"><hr></div></div>';
			}
			
			if($atts['show_pagination'] === 'yes')
			{
				if($events['max_num_pages'] > 1)
				{
					$html .= '<div class="bin_event_calendar_pagination">';
						$html .= paginate_links([
							'type' => 'list',
							'base' => add_query_arg('event-page', '%#%'),
							'format' => '',
							'mid_size' => $atts['pagination_mid_size'],
							'prev_text' => $atts['pagination_prev_text'],
							'next_text' => $atts['pagination_next_text'],
							'screen_reader_text' => $atts['pagination_screen_reader_text'],
							'total' => $events['max_num_pages'],
							'current' => isset($_GET['event-page']) ? abs((int)$_GET['event-page']) : 1
						]);
					$html .= '</div>';
				}
			}
		}
		
		wp_reset_postdata();
		
		if(empty($html)) { return __('No events found...', 'bin-event-calendar'); }
		return $html;
	}
}